#!/usr/bin/env python


__author__ = "Ricardo Fernandes"
__email__ = "ricardo.fernandes@cern.ch"
__copyright__ = "(C) 2011 European Organization for Nuclear Research, Geneva, Switzerland"
__version__ = "1.3"
__date__ = "$Date: 2012-03-08 14:14:57 +0100 (Thu, 08 Mar 2012) $"
__description__ = "Plugin for FlexExtractor to handle SPEC BINARY files"
__status__ = "Production"




# =============================================
#  FUNCTION RESPONSIBLE FOR READING DATA FROM A CERTAIN
#  INPUT. THIS FUNCTION IS CALLED BY FlexExtractor.
# =============================================
def getData(pFile, pSourceList):
    """
    This function reads data from an input source specified in the parameter "pFile". The structure
    defined in "pSourceList" is used to know what data to read from the input source. This function
    returns a tuple with four elements: code, header, data and message.

    Parameter(s):
       pFile - string containing the name of the input source to be read.     
       pSourceList - list containing the input structure that was specified in the XML config.
    
    Return(s):
       int - 0 (the input source was successfully read) or -1 (error found when reading the input source).
       list - contains a list with the headers of the input source (this list should be empty when the "autoselect"
              is False and filled-up when "autoselect" is True).                    
       list - contains the data that was read from the input source (this list will be empty if some error was found).
       list - contains a list of messages that were generated during the reading of the input source.             
    """

    try:
        import FlexExtractor
    except:
        return -1, ["Error when importing module 'FlexExtractor'."] 
    
    try:
        import os
    except:
        return -1, [], [], ["Error when importing module 'os'."]
    
    try:
        import re
    except:
        return -1, [], [], ["Error when importing module 're'."]
    
    try:
        import xlrd
    except:
        return -1, [], [], ["Error when importing module 'xlrd'."]    

    if not os.path.isfile(pFile):
        return -1, [], [], ["The input file '%s' does not exists." % pFile]
      
    try:       
        workBook = xlrd.open_workbook(pFile, formatting_info = True)
    except Exception, e:
        if isinstance(e, xlrd.biffh.XLRDError):
            return -1, [], [], ["The input file '%s' has an unexpected format (check if the 'type' parameter is correctly defined)." % pFile]
        else:
            return -1, [], [], ["Error when reading input file '%s'." % pFile, FlexExtractor.errorLogger()]

    autoSelectList = []
    messageList = []
    result = []               
    for source in pSourceList:
        
        try:
            sheet = workBook.sheet_by_name(source["name"])
        except:
            sheet = None
        
        if sheet is None:
            autoSelectList.append([])
            result.append([])
            messageList.append("The input file '%s' does not have a sheet '%s'." % (pFile, source["name"]))            
        else:
            
            try:

                sourceAutoSelect = source["autoselect"]
                flag = True
                if sourceAutoSelect is None:
                    autoSelectFlag = False
                    startHeader = 6
                    endHeader = 9                    
                else:
                    autoSelectFlag = True
                    if sourceAutoSelect.upper() == "TRUE":                               
                        startHeader = 6
                        endHeader = 9
                    else:
                        try:           
                            sourceAutoSelectList = sourceAutoSelect.split(":", 2)
                            if len(sourceAutoSelectList) == 1:
                                startHeader = int(sourceAutoSelectList[0]) - 1
                                endHeader = int(sourceAutoSelectList[0])
                            else:
                                startHeader = int(sourceAutoSelectList[0]) - 1
                                endHeader = int(sourceAutoSelectList[1])                                                        
                        except:
                            flag = False
                            autoSelectList.append([])
                            messageList.append("The sheet '%s' has an invalid autoselect '%s' (it should specify the start/end of the header rows)." % (source["name"], sourceAutoSelect))

                tmpData0 = []
                if flag:

                    # =============================================
                    #  GET HEADER
                    # =============================================                    
                    mergeList = sheet.merged_cells
                    tmpHeaderList0 = []
                    for i in range(startHeader, endHeader):
                        valueList = sheet.row_values(i)
                        tmpHeaderList1 = []                    
                        j = 0
                        while (j < len(valueList)):
                            mergeAcross = 1
                            for rowLow, rowHigh, colLow, colHigh in mergeList:                           
                                if i == rowLow and j == colLow:
                                    mergeAcross = colHigh - colLow
                                    break
                            for k in range(mergeAcross):
                                tmpHeaderList1.append(valueList[j])
                            j += mergeAcross
                        tmpHeaderList0.append(tmpHeaderList1)
                    tmpAutoSelectList = []             
                    headerList = {}                
                    if tmpHeaderList0 != []:
                        for i in range(len(tmpHeaderList0[0])):
                            header = ""
                            for j in range(len(tmpHeaderList0)):
                                if header == "":
                                    header = "%s" % tmpHeaderList0[j][i]
                                else:
                                    if tmpHeaderList0[j][i] != "":
                                        header = "%s.%s" % (header, tmpHeaderList0[j][i])                                            
                            headerList[header] = i
                            for data in source["data"]:
                                if data["regexpselect"] is not None and data["regexpselect"].upper() == "TRUE":
                                    try:
                                        if re.search(data["select"], header) is not None:
                                            data["select"] = header
                                    except:
                                        pass                                    
                            tmpAutoSelectList.append(header)
                    autoSelectList.append(tmpAutoSelectList)                   
                    
                    
                    # =============================================
                    #  GET DATA
                    # =============================================
                    for i in range(sheet.nrows):
                        valueList = sheet.row_values(i)
                        typeList = sheet.row_types(i)                    
                        tmpData1 = []
                        for j in range(len(valueList)):
                            value = valueList[j]
                            if typeList[j] != xlrd.XL_CELL_NUMBER or (value - int(value)) != 0:
                                tmpData1.append(value)
                            else:
                                tmpData1.append(int(value))                                
                        if autoSelectFlag:
                            tmpData2 = tmpData1
                        else:
                            tmpData2 = []
                        for data in source["data"]:
                            if data["select"] is None:
                                tmpData2.append(None)
                            else:                                     
                                try:
                                    tmpData2.append(tmpData1[headerList[data["select"]]])
                                except:
                                    message = "The input file '%s' does not have a select '%s' in sheet '%s' or it is invalid." % (pFile, data["select"], source["name"])
                                    if message not in messageList:
                                        messageList.append(message)                                
                                    tmpData2.append(None)
                        tmpData0.append(tmpData2)                                                        
                                        
                result.append(tmpData0)
                
            except MemoryError:
                return -1, [], [], ["Error when reading input file '%s' since there is not enough memory available." % pFile]                                                                
            except:
                result.append([])
                messageList.append("An unexpected error occurred when processing sheet '%s'." % source["name"], FlexExtractor.errorLogger())
            
    return 0, autoSelectList, result, messageList    




# =============================================
#  FUNCTION RESPONSIBLE FOR WRITING DATA TO A CERTAIN
#  OUTPUT. THIS FUNCTION IS CALLED BY FlexExtractor.
# =============================================
def setData(pFile, pParameter, pDataList, pAppendFlag):
    """
    This function writes the data "pDataList" into a certain output specified in the parameter "pFile"
    and uses the parameter "pParameter" to provide additional information on how to do it. If "pAppendFlag"
    is True then the data will be appended in the end of the output, otherwise (if False) the previous
    data will be erased before writing the new data. This function returns a tuple with two elements:
    code and message.

    Parameter(s):
       pFile - string containing the name of the output where to write the data.    
       pParameter - string containing the parameter that was specified in the XML config.    
       pDataList - list containing the data to be written.
       pAppendFlag - boolean that specifies if the output is to be overwritten (True) or appended (False) with the new data.
    
    Return(s):
       int - 0 (the data was successfully written) or -1 (error found when writing the data).             
       list - contains a list of messages that were generated during the writing of the data.      
    """
    
    return 0, []
    

