#!/usr/bin/env python


__author__ = "Ricardo Fernandes"
__email__ = "ricardo.fernandes@cern.ch"
__copyright__ = "(C) 2011 European Organization for Nuclear Research, Geneva, Switzerland"
__version__ = "1.4"
__date__ = "$Date: 2012-02-03 20:54:05 +0100 (Fri, 03 Feb 2012) $"
__description__ = "Plugin for FlexExtractor to handle TXT files"
__status__ = "Production" 




# =============================================
#  FUNCTION RESPONSIBLE FOR READING DATA FROM A CERTAIN
#  INPUT. THIS FUNCTION IS CALLED BY FlexExtractor.
# =============================================
def getData(pFile, pSourceList):
    """
    This function reads data from an input source specified in the parameter "pFile". The structure
    defined in "pSourceList" is used to know what data to read from the input source. This function
    returns a tuple with four elements: code, header, data and message.

    Parameter(s):
       pFile - string containing the name of the input source to be read.     
       pSourceList - list containing the input structure that was specified in the XML config.
    
    Return(s):
       int - 0 (the input source was successfully read) or -1 (error found when reading the input source).
       list - contains a list with the headers of the input source (this list should be empty when the "autoselect"
              is False and filled-up when "autoselect" is True).                    
       list - contains the data that was read from the input source (this list will be empty if some error was found).
       list - contains a list of messages that were generated during the reading of the input source.             
    """

    try:
        import os
    except:
        return -1, [], [], ["Error when importing module 'os'."]
    
    try:
        import re
    except:
        return -1, [], [], ["Error when importing module 're'."]    
    
    if not os.path.isfile(pFile):
        return -1, [], [], ["The input file '%s' does not exists." % pFile]   

    try:

        autoSelectList = []
        messageList = []
        result = []
        fileHandler = open(pFile, "r")
        lineList = fileHandler.readlines()
        fileHandler.close()        
        
        for source in pSourceList:
            
            sourceParameter = source["parameter"]
            if sourceParameter is not None:
                parameterList = []
                lastPosition = 0
                parameter = ""
                while(True):
                    i = sourceParameter.find(";", lastPosition)
                    if i == -1:
                        parameterList.append("%s%s" % (parameter, sourceParameter[lastPosition:]))
                        break
                    else:
                        if i == 0 or sourceParameter[i - 1] != "\\":
                            parameterList.append("%s%s" % (parameter, sourceParameter[lastPosition:i]))
                            parameter = ""
                        else:
                            parameter = "%s%s;" % (parameter, sourceParameter[lastPosition:i - 1])
                        lastPosition = i + 1
                if "" in parameterList:
                    return -1, [], [], ["The source 'parameter' has one or more separators with an empty value."]
            
            
            # =============================================
            #  GET HEADER
            # =============================================
            sourceAutoSelect = source["autoselect"]
            flag = True
            if sourceAutoSelect is None:
                autoSelectFlag = False
            else:
                autoSelectFlag = True
                if sourceAutoSelect.upper() == "TRUE":                               
                    startHeader = 0
                    endHeader = 1
                else:
                    try:           
                        sourceAutoSelectList = sourceAutoSelect.split(":", 2)
                        if len(sourceAutoSelectList) == 1:
                            startHeader = int(sourceAutoSelectList[0]) - 1
                            endHeader = int(sourceAutoSelectList[0])
                        else:
                            startHeader = int(sourceAutoSelectList[0]) - 1
                            endHeader = int(sourceAutoSelectList[1])                                                        
                    except:
                        flag = False
                        autoSelectList.append([])
                        messageList.append("The source '%s' has an invalid autoselect '%s' (it should specify the start/end of the header rows)." % (source["name"], sourceAutoSelect))
                if flag:
                    tmpHeaderList = []
                    for line in lineList[startHeader:endHeader]:
                        if line[-1] == "\n": 
                            valueList = [line[:-1]]
                        else:
                            valueList = [line]
                        if sourceParameter is not None:
                            for parameter in parameterList:
                                value, valueList = valueList, []
                                for sequence in value:
                                    valueList += sequence.split(parameter)
                        tmpHeaderList.append(valueList)
                    tmpAutoSelectList = []             
                    headerList = {}                
                    if tmpHeaderList != []:
                        for i in range(len(tmpHeaderList[0])):
                            header = ""
                            for j in range(len(tmpHeaderList)):
                                if header == "":
                                    header = "%s" % tmpHeaderList[j][i]
                                else:
                                    if tmpHeaderList[j][i] != "":
                                        header = "%s.%s" % (header, tmpHeaderList[j][i])
                            headerList[header] = i
                            for data in source["data"]:
                                if data["regexpselect"] is not None and data["regexpselect"].upper() == "TRUE":
                                    try:
                                        if re.search(data["select"], header) is not None:
                                            data["select"] = header
                                    except:
                                        pass
                            tmpAutoSelectList.append(header)
                    autoSelectList.append(tmpAutoSelectList)                                        
                    
            tmpData0 = []            
            if flag:
                for line in lineList:
                    if line[-1] == "\n": 
                        valueList = [line[:-1]]
                    else:
                        valueList = [line]                        
                    if sourceParameter is not None:
                        for parameter in parameterList:
                            value, valueList = valueList, []
                            for sequence in value:
                                valueList += sequence.split(parameter)                                
    
                    # =============================================
                    #  GET DATA
                    # =============================================
                    valueLength = len(valueList)
                    if autoSelectFlag:
                        tmpData1 = valueList
                    else:
                        tmpData1 = []
                    for data in source["data"]:
                        if data["select"] is None:
                            tmpData1.append(None)
                        else:                        
                            if autoSelectFlag:
                                try:
                                    tmpData1.append(valueList[headerList[data["select"]]])
                                except:
                                    message = "The input file '%s' does not have a select '%s' in source '%s' or it is invalid." % (pFile, data["select"], source["name"])
                                    if message not in messageList:
                                        messageList.append(message)                                
                                    tmpData1.append(None)
                            else:
                                try:
                                    index = int(data["select"])
                                except:
                                    select = data["select"].upper() 
                                    index = 0
                                    j = len(select)
                                    for k in select:
                                        j -= 1
                                        index += j * 25 + ord(k) - 64
                                if index > valueLength:
                                    message = "The input file '%s' does not have a select '%s' in source '%s' or it is invalid." % (pFile, data["select"], source["name"])
                                    if message not in messageList:
                                        messageList.append(message)                                
                                    tmpData1.append(None)
                                else:
                                    tmpData1.append(valueList[index - 1])
                    tmpData0.append(tmpData1)                
                    
            result.append(tmpData0)
            
        return 0, autoSelectList, result, messageList
        
    except MemoryError:
        return -1, [], [], ["Error when reading input file '%s' since there is not enough memory available." % pFile]
    except:
        return -1, [], [], ["Error when reading input file '%s'." % pFile]
    




# =============================================
#  FUNCTION RESPONSIBLE FOR WRITING DATA TO A CERTAIN
#  OUTPUT. THIS FUNCTION IS CALLED BY FlexExtractor.
# =============================================
def setData(pFile, pParameter, pDataList, pAppendFlag):
    """
    This function writes the data "pDataList" into a certain output specified in the parameter "pFile"
    and uses the parameter "pParameter" to provide additional information on how to do it. If "pAppendFlag"
    is True then the data will be appended in the end of the output, otherwise (if False) the previous
    data will be erased before writing the new data. This function returns a tuple with two elements:
    code and message.

    Parameter(s):
       pFile - string containing the name of the output where to write the data.    
       pParameter - string containing the parameter that was specified in the XML config.    
       pDataList - list containing the data to be written.
       pAppendFlag - boolean that specifies if the output is to be overwritten (True) or appended (False) with the new data.
    
    Return(s):
       int - 0 (the data was successfully written) or -1 (error found when writing the data).             
       list - contains a list of messages that were generated during the writing of the data.      
    """
    
    return 0, []
    


