package org.xdv.clx.builder;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xdv.common.UtilDOM;

/**
 * Helper to access elements/attributes in W3C DOM XML document.
 * It contains set of methods that are useful when creating CLiXML rule from 
 * W3C DOM XML source (see {@link ClxDOMBuilder}).
 * <p>
 * Helper is always associated with namespace.  
 * This particular version is associated with <i>empty</i> namespace. In other words - it operates 
 * on nodes where namespaceURI is empty.
 * <br>
 * There is an extension to this class that may work with any given namespace URI
 * - {@link org.xdv.clx.builder.ClxDOMHelperNS}. 
 * </p>
 * <p>
 * To create an instance call {@link #newInstance()}.
 * </p>
 *
 * @author Tomasz Kokoszka
 */
public class ClxDOMHelper {
    
    /** 
     * Singleton - created when class is initialized, returned by {@link #newInstance()}.
     */
    private static final ClxDOMHelper SINGLETON = new ClxDOMHelper(); 
    
    /**
     * Returns instance of helper that works with <i>empty</i> namepsace.
     * As such a helper doesn't have a state, this method always returns the same object.
     * @return a helper to work with <i>empty</i> namespace
     */
    public static ClxDOMHelper newInstance() {
        return SINGLETON;
    }
    
    /**
     * Creates new instance.
     */
    protected ClxDOMHelper() {
    }
    
    /**
     * Returns namespaceURI that this helper is configured to work with.
     * @return namespace URI; can be null
     */
    protected String getNS() {
        return null;
    }
    
    /** 
     * Checks if given namespace URI is the same namespace as the namespace that this helper is
     * configured to use.
     * @param namespaceURI namespace URI to check; can be null
     * @return true if this helper is confugred to exacly the same namespace
     */
    public boolean isNS(String namespaceURI) {
        return UtilDOM.isEmptyNS(namespaceURI);
    }
    
    /**
     * Returns node name - local name, a name without namespace prefix.
     * @param node W3C DOM node
     * @return node name; <code>null</code> if <code>node</code> is not an element nor attribute
     */
    public String getNodeName(Node node) {
        return node.getNodeName();
    }
    
    /**
     * Returns direct children with given node name and helper's namespace.
     * It skips children with other name or from other namespace than namespace applicable for 
     * this helper.
     * @param root subtree root
     * @param tagName XML node name (without namespace prefix, only local name)
     * @return list of {@link Element}; !=null
     */
    public List getChildren(Element root, String tagName) {
        final NodeList nl = root.getChildNodes();
        final int size = nl.getLength();
        final List result = new ArrayList();
        for(int i = 0; i < size; i++) {
            final Node n = nl.item(i);
            if ((n.getNodeType() == Node.ELEMENT_NODE) &&
                (isNS(n.getNamespaceURI())) &&
                (tagName.equals(getNodeName(n)))) {
                result.add(n);
            }
        }
        
        return result;
    }

    /**
     * Returns direct children that are in helper's namespace. 
     * @param root subtree root
     * @return list of {@link Element}; !=null
     * @throws ClxBuilderException found element from unexpected namespace
     */
    public List getChildren(Element root)
    throws ClxBuilderException {
        final NodeList nl = root.getChildNodes();
        final int size = nl.getLength();
        final List result = new ArrayList();
        for(int i = 0; i < size; i++) {
            final Node n = nl.item(i);
            if (n.getNodeType() == Node.ELEMENT_NODE) {
                if (isNS(n.getNamespaceURI())) {
                    result.add(n);
                } else {
                    throw new ClxBuilderException(
                            "Invalid namespace - " + n.getNamespaceURI() + ", only " +
                            getNS() + "is accepted", UtilDOM.getSimplePath(n));                    
                }
            }
        }
        
        return result;
    }
    
    /**
     * Returns direct children that are in helper namespace or in <code>acceptNS</code> or
     * an <i>empty</i> namespace (only when <code>acceptEmptyNS</code> is true). 
     * @param root subtree root
     * @param acceptNS namespace URI; can be null
     * @param acceptEmptyNS accept also elements from <i>empty</i> namespace
     * @return list of {@link Element}; !=null
     * @throws ClxBuilderException
     */
    public List getChildrenWithNS(Element root, String acceptNS, boolean acceptEmptyNS)
    throws ClxBuilderException {
        final NodeList nl = root.getChildNodes();
        final int size = nl.getLength();
        final List result = new ArrayList();
        for(int i = 0; i < size; i++) {
            final Node n = nl.item(i);
            final String ns = n.getNamespaceURI();
            if (n.getNodeType() == Node.ELEMENT_NODE) {
                if (isNS(ns) || UtilDOM.isNS(ns, acceptNS, acceptEmptyNS)) {
                    result.add(n);
                } else {
                    throw new ClxBuilderException("Invalid namespace - " + ns + ", only " +
                            getNS() + " is accepted", UtilDOM.getSimplePath(n));
                }
            }
        }
        
        return result;
    }
    
    /**
     * Returns first and only one direct child that has given node name.
     * <br>
     * Method should be used when at most one element is expected.
     * @param root subtree root
     * @param tagName XML node name (without namespace prefix, only local name)
     * @return first direct child with given name or <code>null</code> if there is no such element
     * @throws ClxBuilderException there are more than one direct child with this name          
     */
    public Element getFirstOne(Element root, String tagName) 
    throws ClxBuilderException {
        final NodeList nl = root.getChildNodes();
        final int size = nl.getLength();
        Element result = null;

        for(int i = 0; i < size; i++) {
            final Node n = nl.item(i);
            if ((n.getNodeType() == Node.ELEMENT_NODE) &&
                (isNS(n.getNamespaceURI())) &&
                (tagName.equals(getNodeName(n)))) {
                if (result != null) {
                    throw new ClxBuilderException("At most one " + tagName + " expected", root.getNodeName());
                }
                result = (Element) n;
            }
        }
        
        return result;
    }
            
    /**
     * Returns attribute value.
     * <br>
     * Method should be used you expect that attribute exists and has no empty value.
     * @param root element - method gets attribute from this element and returns its value
     * @param attrName attribute name
     * @return attribute value; !=null
     * @throws ClxBuilderException attribute doesn't exists or has an empty value
     */
    public String getAttrValueNoEmpty(Element root, String attrName) 
    throws ClxBuilderException {
        final String v = getAttrValue(root, attrName, null);
        if ((v == null) || (v.trim().length() == 0)) {
            throw new ClxBuilderException("Attribute " + attrName + " is missing or empty", 
                    UtilDOM.getSimplePath(root) + "/@" + attrName);
        }
        return v;
    }
    
    /**
     * Returns attribute value.
     * @param root element - method gets attribute from this element and returns its value
     * @param attrName attribute name
     * @param defaultValue default value returned when attribute doesn't exist
     * @return attribute value or <code>defaultValue</code> if attribute is not defined
     */
    public String getAttrValue(Element root, String attrName, String defaultValue) {
        return UtilDOM.getAttributeValue(root, attrName, defaultValue);
    }            
                
}
