package org.xdv.clx.extension;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.xdv.clx.base.ClxValidationError;
import org.xdv.clx.exec.ClxExecutionContext;
import org.xdv.clx.expr.ClxRule;
import org.xdv.common.XdvValidationError;
import org.xdv.common.XdvValidationException;
import org.xdv.xpath.XPathException;
import org.xdv.xpath.XPathExpression;
import org.xdv.xpath.XPathValue;
import org.xdv.xpath.XPathValueNodeSet;

/**
 * Error reporter - CLX extended element implementation.
 * <p>
 * This element is dedicated to generate validatoion-error object.
 * It is an extension to standard way of reporting errors defined in CLiXML 1.0. 
 * Extension is defined in <code>clix_Report.xsd</code>.
 * </p>
 * 
 * @see <a href="http://www.clixml.org/clix/1.0/#reporting">CLiXML - clix:report</a>
 * @see org.xdv.clx.base.ClxConstants#TAG_EXT_ERRORREPORT
 * @author Tomasz Kokoszka
 */
public class ClxExtErrorReport {
        
    /** Error level for all validation-errors reported by this reporter.
     * {@link XdvValidationError#LEVEL_ERROR}, {@link XdvValidationError#LEVEL_WARNING} 
     */
    private final int errorLevel;
    
    /**
     * List of object to construct validation-error message.
     * List of {@link ClxExtErrorMessage}; !=null.
     */
    private final List messageTexts;
    
    /**
     * List of XPath expressions to get nodes that <i>caused</i> error.
     * List of {@link XPathExpression}; !=null.
     */
    private final List errorNodes;
    
    /**
     * Create error reporter with simple text message.
     * Validation-errors generated by this reporter will be at level ERROR.
     * @param message text message
     */
    public ClxExtErrorReport(String message) {
        this.errorLevel = XdvValidationError.LEVEL_ERROR;        
        this.errorNodes = new ArrayList(0);
        this.messageTexts = new ArrayList(1);
        this.messageTexts.add(new ClxExtErrorMessageText(message));
    }
        
    /**
     * Creates error reporter.
     * @param errorLevel error level: {@link XdvValidationError#LEVEL_ERROR}, 
     *        {@link XdvValidationError#LEVEL_WARNING}
     * @param messageTexts list of object to construct validation-error message. List of
     *        {@link ClxExtErrorMessage}
     * @param errorNodes list of XPath expressions to get nodes that <i>caused</i> error.
     *        List of {@link XPathExpression}
     */
    public ClxExtErrorReport(int errorLevel, List messageTexts, List errorNodes) {
        this.errorLevel = errorLevel;
        this.messageTexts = messageTexts;
        this.errorNodes = errorNodes;
    }
    
    /**
     * Returns error level for all errors generated by this reporter.
     * @return error level: {@link XdvValidationError#LEVEL_ERROR}, 
     *         {@link XdvValidationError#LEVEL_WARNING}
     */
    public int getErrorLevel() {
        return errorLevel;
    }
    
    /**
     * Generates new validation-error.
     * Created error will have:
     * <ul>
     *  <li>error level = {@link #getErrorLevel()}
     *  <li>message dynamically created from internal list
     *  <li>nodes that caused the problem (if they were defined)
     * </ul>
     * @param cntx CLX execution context (to execute some XPath expressions)
     * @param failedRule rule that failed (rule that is not valid and wants to generate 
     *        validation-error using this reporter)
     * @return validation-error; !=null
     * @throws XdvValidationException error while creating error (invalid xpath expression)
     */
    public ClxValidationError generateError(ClxExecutionContext cntx, ClxRule failedRule)
    throws XdvValidationException {
        //message
        StringBuffer message = new StringBuffer();
        for(Iterator i = messageTexts.iterator(); i.hasNext(); ) {
            final ClxExtErrorMessage msgGen = (ClxExtErrorMessage) i.next();
            message.append(msgGen.generateMessage(cntx));
        }

        final ClxValidationError error = new ClxValidationError(this.errorLevel, message.toString());
        
        //rule that failed
        error.setRule(failedRule);
                
        //error nodes
        error.setDocumentType(cntx.getXPathEngine().getDocumentClass());
        for(Iterator i = errorNodes.iterator(); i.hasNext(); ) {
            final XPathExpression xpath = (XPathExpression) i.next();
            try {
                final XPathValue set = cntx.executeXPath(xpath);
                if (set.getType() == XPathValue.TYPE_NODESET) {
                    for(Iterator j = ((XPathValueNodeSet) set).getNodes().iterator(); j.hasNext(); ) {
                        final Object node = j.next();
                        error.addNode(node);
                    }
                }
            } catch (XPathException e) {
                throw new XdvValidationException("Xpath execution error while making list of " +
                        "invalid nodes (errorNode) - xpath " + xpath);
            }            
        }
                
        return error;
    }
    
    public String toString() {
        StringBuffer sb = new StringBuffer();
        if (errorLevel == XdvValidationError.LEVEL_ERROR) {
            sb.append("ERROR");
        } else if (errorLevel == XdvValidationError.LEVEL_WARNING) {
            sb.append("WARNING");
        } else {
            sb.append(errorLevel);
        }
        sb.append(": ");
        
        for(Iterator i = messageTexts.iterator(); i.hasNext(); ) {
            sb.append(i.next());
        }
        
        return sb.toString();
    }
        
}
