package org.xdv.xpath;

/**
 * Interface to execute set of XPath expressions in the same context 
 * (variables, XML document, current node, etc).
 * <p>
 * Describes all execution parameters like XML document, variable mapping, 
 * XPath current node (self), etc.
 * Provides functionality to:
 * <ol class="NoTopMargin">
 * <li>Set current node (axis <i>self</i> in XPath) - 
 *      {@link #setContext(Object)}, {@link #setContextRoot()}.
 * <li>Manage XPath variables - {@link #getVariables()}, {@link #setVariables(XPathVariableMap)}.
 *     <br><b>Note:</b> Method {@link #getVariables()} have to return exacly the same variable map 
 *     that is used to resolve variables when executing XPath expression. 
 *     It means that modifications done in variable map obtained by {@link #getVariables()} 
 *     influents further executions. 
 *     <br>For instance: if user will set a new variable, this new value will be used in all further 
 *     execution (as far as user will not set a new value or variable map).
 * <li>Execute XPath expression - {@link #execute(XPathExpression)}, 
 *     {@link #executeSelect(XPathExpression)}.
 * </ol>    
 * Instance should be created using {@link XPathEngine#createExecutionUnit(Object)}.
 * Once created, it is always binded to one XML document and this binding cannot be changed.
 * To execute the same XPath expression in different XML document, a new execution unit has to be
 * created.  
 * </p>
 * <p>
 * <b>Note:</b>usually execution unit works only with expressions created by the same 
 * instance of engine as execution unit. See section '<i>Assumptions and limitations</i>' in 
 * package description ({@link org.xdv.xpath}) for more details.
 * </p>  
 * <p>
 * XPath execution unit is not thread safe - use different execution unit in every thread.
 * </p>
 *
 * @author Tomasz Kokoszka
 */
public interface XPathExecutionUnit {
    
    /**
     * Returns engine instance that created this execution-unit.
     * @return XPath engine; !=null
     */
    public XPathEngine getEngine();
    
    /**
     * Sets <i>current</i> (axis <i>self</i>) element.
     * @param contextNative current node, it must be a XML node
     * @throws XPathInvalidTypeError invalid type of XML node
     */
    public void setContext(Object contextNative)
    throws XPathInvalidTypeError;
    
    /**
     * Sets <i>current</i> (axis <i>self</i>) element) element to document root.   
     */
    public void setContextRoot();
    
    /**
     * Returns variable map.
     * It is the same variable map that is used to resolve variables when executing 
     * XPath expression. 
     * It means that any modifications done to returned map influents further execution. 
     * For instance to add a new variable to execution unit:
     * <pre>
     *  XPathExecutionUnit unit;
     *  XPathValue value;
     *  ...
     *  //create unit and value using XPath engine
     *  ... 
     *  unit.getVariables().setVariable('varName', value);
     * </pre>
     * @return Variable map; !=null
     */
    public XPathVariableMap getVariables();
    
    /**
     * Sets new variable map.
     * @param newVariables the variable map. This map will be used to resolve variables
     *        when executing XPath expressions
     */    
    public void setVariables(XPathVariableMap newVariables)
    throws XPathInvalidTypeError;
    
    /**
     * Executes XPath expression that return set of nodes.
     * <br>
     * Given XPath expression must return a node-set, otherwise a {@link XPathException} will be 
     * thrown. 
     * For instance <code>/data/field/@name</code> returns a text and cannot be executed properly by
     * this method.
     * @param expression XPath expression to execute.         
     * @return XPath result; !=null
     * @throws XPathException error while executing xpath expression
     * @throws XPathInvalidTypeError given XPath expression not valid for this execution unit
     */
    public XPathValueNodeSet executeSelect(XPathExpression expression) 
    throws XPathException, XPathInvalidTypeError;
    
    /**
     * Executes any XPath expression. 
     * @param expression XPath expression to execute. Usually execution unit works only with
     *        expressions created by the same engine.
     * @return wrapped XPath result; !=null
     * @throws XPathException error while executing xpath expression
     * @throws XPathInvalidTypeError given XPath expression not valid for this execution unit
     */
    public XPathValue execute(XPathExpression expression)
    throws XPathException, XPathInvalidTypeError;
        
}
